{ TLed version 1.0 copyright  1997 by Alexander Meeder (ameeder@dds.nl)

  This is a visual component to simulate a LED. You may use it for free but I do not guarantee
  anything

  properties:
  * About       -> read-only property which shows some info. about this version and programmer
  * Alignment   -> default 'laCenter', other options (laBottom, laCenter, laLeft, laRight, laTop)
  * BorderColor -> default 'clGreen', this property is only used when Ctl3D is false
  * BorderStyle -> default 'bsRaised', this porperty is only used when Ctl3D is true
  * BorderWidth -> default value 2 (value must be between 1 and 5, use for the best results with
                   LedShape = lsRndRect or lsRound a value of 2 or higher, with BorderWidth of 1
                   the led isn't drawn right
  * Caption     -> default value 'Led'
  * Ctl3D       -> same as other controls, default is true (draws phong when
                   LedShape = lsRndRect or lsRound)
  * Cursor      -> same as other controls
  * Font        -> same as other controls
  * Height      -> height of LED-control (Attention: not the height of the LED) default value 42
  * Hint        -> same as other controls
  * Interval    -> interval of internal timer (0 to MaxInt), default 0. To save resources
                   there is no Timer created when value is 0.
  * LedHeight   -> default value 10, (value must be between 5 and 100)
  * LedOn       -> boolean, default false. Read/Write property (exampl. Led1.LedOn := not Led1.LedOn)
  * LedShape    -> default lsRect, you have three choices lsRect, lsRndRect and lsRound
  * LedWidth    -> default value 25, (value must be between 5 and 100)
  * Left        -> same as other controls
  * Margin      -> default value 3, margin between LED and caption (value must be between 1 and 10)
  * Name        -> same as other controls, default value Ledxx - where xx unique identifier
  * OffColor    -> default is clGreen
  * OnColor     -> default is clLime
  * PhongSize   -> default 2, value must be between 1 and 5
  * ShowHint    -> same as other controls, default value false
  * Tag         -> same as other controls
  * Top         -> same as other controls
  * Width       -> same as other controls, default value 66

  events:
  * OnChange    -> when Led-status changes this event occur, to determine if Led is 'on' or
                   'off' you can use 'IsOn'-parameter (true=on / false=off)

  If you have suggestions about TLed or make changes, please mail me: ameeder@dds.nl
}

unit AMLed;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls, dsgnIntf;

type
  TInterval = 0..MaxInt;
  TBorderStyle = (bsRaised, bsLowered);
  TBrdrWidth = 1..5;
  TLedDimension = 5..50;
  TLedShape = (lsRect, lsRndRect, lsRound);
  TMargin = 1..10;
  TMyAlignment = (laLeft, laRight, laCenter, laTop, laBottom);
  TOnOff = procedure (Sender : TObject; IsOn : Boolean) of object;

  TAboutProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  TLed = class(TGraphicControl)
  private
    FAbout: TAboutProperty;
    FAlignment: TMyAlignment;
    FBorderColor: TColor;
    FBorderStyle: TBorderStyle;
    FBorderWidth: TBrdrWidth;
    FCaption: TCaption;
    FCtl3D: boolean;
    FFont: TFont;
    FInterval: TInterval;
    FLedHeight: TLedDimension;
    FLedOn: boolean;
    FLedShape: TLedShape;
    FLedWidth: TLedDimension;
    FMargin: TMargin;
    FOffColor: TColor;
    FOnColor: TColor;
    FOnOff: TOnOff;
    FPhongSize: TBrdrWidth;
    FTimer: TTimer;

    LedRect: TRect;
    LedOrigin: TPoint;
    CaptionOrigin: TPoint;

    procedure SetAlignment(value: TMyAlignment);
    procedure SetBorderStyle(value: TBorderStyle);
    procedure SetBorderWidth(value: TBrdrWidth);
    procedure SetCaption(value: TCaption);
    procedure SetColor(index: integer; value: TColor);
    procedure SetCtl3D(value: boolean);
    procedure SetFont(value: TFont);
    procedure SetInterval(value: TInterval);
    procedure SetLedDimension(index: integer; value: TLedDimension);
    procedure SetLedOn(value: boolean);
    procedure SetLedShape(value: TLedShape);
    procedure SetMargin(value: TMargin);
    procedure SetPhongSize(value: TBrdrWidth);
  protected
    procedure Paint; override;
    procedure Loaded; override;
    procedure TimerTimer(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About: TAboutProperty read FAbout write FAbout;
    property Alignment: TMyAlignment read FAlignment write SetAlignment default laCenter;
    property BorderColor: TColor index 3 read FBorderColor write SetColor default clGreen;
    property BorderStyle: TBorderStyle read FBOrderStyle write SetBorderStyle default bsRaised;
    property BorderWidth: TBrdrWidth read FBorderWidth write SetBorderWidth default 2;
    property Caption: TCaption read FCaption write SetCaption;
    property Ctl3D: boolean read FCtl3D write SetCtl3D default true;
    property Font: TFont read FFont write SetFont;
    property Interval: TInterval read FInterval write SetInterval default 0;
    property LedHeight: TLedDimension index 1 read FLedHeight write SetLedDimension default 10;
    property LedOn: boolean read FLedon write SetLedOn default false;
    property LedShape: TLedShape read FLedShape write SetLedShape default lsRect;
    property LedWidth: TLedDimension index 2 read FLedWidth write SetLedDimension default 25;
    property Margin: TMargin read FMargin write SetMargin default 3;
    property OffColor: TColor index 1 read FOffColor write SetColor default clGreen;
    property OnColor: TColor index 2 read FOnColor write SetColor default clLime;
    property PhongSize: TBrdrWidth read FPhongSize write SetPhongSize default 2;
    property ShowHint;
    property OnChange: TOnOff read FOnOff write FOnOff;
  end;

procedure Register;

implementation

procedure TLed.SetAlignment(value: TMyAlignment);
begin
  if value <> FAlignment then
  begin
    FAlignment := value;
    invalidate;
  end;
end;

procedure TLed.SetBorderStyle(value: TBorderStyle);
begin
  if value <> FBorderStyle then
  begin
    FBorderStyle := value;
    invalidate;
  end;
end;

procedure TLed.SetBorderWidth(value: TBrdrWidth);
begin
  if value <> FBorderWidth then
  begin
    FBorderWidth := value;
    invalidate;
  end;
end;

procedure TLed.SetCaption(value: TCaption);
begin
  if value <> FCaption then
  begin
    FCaption := value;
    invalidate;
  end;
end;

procedure TLed.SetCtl3D(value: boolean);
begin
  if value <> FCtl3D then
  begin
    FCtl3D := value;
    invalidate;
  end;
end;

procedure TLed.SetFont(value: TFont);
begin
  if value <> FFont then
  begin
    FFont.Assign(value);
    invalidate;
  end;
end;

procedure TLed.SetInterval(value: TInterval);
begin
  if value <> FInterval then
  begin
    FInterval := value;
    if Assigned(FTimer) then FTimer.Interval := FInterval;
  end;
end;

procedure TLed.SetLedDimension(index: integer; value: TLedDimension);
begin
  case index of
    1: if FLedHeight <> value then FLedHeight := value;
    2: if FLedWidth <> value then FLedWidth := value;
  end;
  if FLedHeight > Height then Height := FLedHeight;
  if FLedWidth > Width then Width := FLedWidth;
  invalidate;
end;

procedure TLed.SetLedOn(value: boolean);
begin
  if value <> FLedOn then
  begin
    FLedOn := value;
    if Assigned(FOnOff) then FOnOff(Self, FLedOn);
    invalidate;
  end;
end;

procedure TLed.SetLedShape(value: TLedShape);
begin
  if value <> FLedShape then
  begin
    FLedShape := value;
    invalidate;
  end;
end;

procedure TLed.SetColor(index: integer; value: TColor);
begin
  case index of
    1: if value <> FOffColor then FOffColor := value;
    2: if value <> FOnColor then FOnColor := value;
    3: if value <> FBorderColor then FBorderColor := value;
  end;
  invalidate;
end;

procedure TLed.SetMargin(value: TMargin);
begin
  if value <> FMargin then
  begin
    FMargin := value;
    invalidate;
  end;
end;

procedure TLed.SetPhongSize(value: TBrdrWidth);
begin
  if value <> FPhongSize then
  begin
    FPhongSize := value;
    invalidate;
  end;
end;

procedure TLed.TimerTimer(Sender: TObject);
begin
  LedOn := not LedOn;
end;

procedure TLed.Loaded;
begin
  if not (csDesigning in ComponentState) then
  begin
    if FInterval > 0 then
    begin
      FTimer := TTimer.Create(self);
      FTimer.Interval := FInterval;
      FTimer.OnTimer := TimerTimer;
    end;
  end;
end;

procedure TLed.Paint;
begin
  with canvas do
  begin
    // This is for redrawing at design-time, first clean old layout...
    if (csDesigning in ComponentState) then
    begin
      brush.style := bsSolid;
      brush.color := color;
      pen.style := psDot;
      pen.color := clGray;
      Rectangle(0, 0, Width, Height);
    end;

    if FLedOn then
      brush.color := FOnColor
    else
      brush.color := FOffColor;

    with Pen do
    begin
      if Ctl3D and (FLedShape = lsRect) then
        color := brush.color
      else
        color := FBorderColor;

      style := psSolid;
      width := FBorderWidth;
    end;

    brush.style := bsSolid;
    font := FFont;

    case FAlignment of
      laBottom: begin
                  LedOrigin.X := (width - FLedWidth) div 2;
                  LedOrigin.Y := height - FLedHeight;
                  CaptionOrigin.X := (width - TextWidth(FCaption)) div 2;
                  CaptionOrigin.Y := (LedOrigin.Y - FMargin - TextHeight(Caption));
                end;
      laCenter: begin
                  LedOrigin.X := (width - FLedWidth) div 2;
                  LedOrigin.Y := (height - FLedHeight) div 2;
                  CaptionOrigin.X := (width - TextWidth(Caption)) div 2;
                  CaptionOrigin.Y := (LedOrigin.Y + FLedHeight + FMargin);
                end;
      laLeft  : begin
                  LedOrigin.X := 1;
                  LedOrigin.Y := (height - FLedHeight) div 2;
                  CaptionOrigin.X := LedOrigin.X + FLedWidth + FMargin;
                  CaptionOrigin.Y := (height - TextHeight(Caption)) div 2;
                end;
      laRight : begin
                  LedOrigin.X := width - FLedWidth;
                  LedOrigin.Y := (height - FLedHeight) div 2;
                  CaptionOrigin.X := (LedOrigin.X - FMargin - TextWidth(Caption));
                  CaptionOrigin.Y := (height - TextHeight(Caption)) div 2;
                end;
      laTop   : begin
                  LedOrigin.X := (width - FLedWidth) div 2;
                  LedOrigin.Y := 1;
                  CaptionOrigin.X := (width - TextWidth(Caption)) div 2;
                  CaptionOrigin.Y := (LedOrigin.Y + FLedHeight + FMargin);
                end;
    end;

    LedRect := Rect(LedOrigin.X, LedOrigin.Y, LedOrigin.X+FLedWidth, LedOrigin.Y+FLedHeight);

    case FLedShape of
      lsRect    : Rectangle(LedRect.Left+1, LedRect.Top+1, LedRect.Right-1, LedRect.Bottom-1);
      lsRndRect : RoundRect(LedRect.Left+1, LedRect.Top+1, LedRect.Right-1, LedRect.Bottom-1, 10, 10);
      lsRound   : Ellipse(LedRect.Left, LedRect.Top, LedRect.Right, LedRect.Bottom);
    end;

    // Draw phong when Ctl3D and lsRndRect of lsRound...
    if Ctl3D and (FLedShape = lsRndRect) or (FLedShape = lsRound) then
    begin
      if FLedOn then
      begin
        Pen.Color := clWhite;
        Pen.Width := 1;
        Brush.Color := clWhite;
        Brush.Style := bsSolid;
      end
      else
      begin
        Pen.Color := clSilver;
        Pen.Width := 1;
        Brush.Color := clSilver;
        Brush.Style := bsSolid;
      end;
      Ellipse((LedRect.Left+(FLedWidth div 3)), (LedRect.Top+(FLedHeight div 3)),
              (LedRect.Left+(FLedWidth div 3)+FPhongSize), (LedRect.Top+(FLedHeight div 3)+FPhongSize));
    end;

    if Caption > '' then
    begin
      brush.color := color;
      TextOut(CaptionOrigin.X, CaptionOrigin.Y, Caption);
    end;

    if Ctl3D and (FLedShape = lsRect) then
    begin
      if InflateRect(LedRect, 1, 1) then
      case FBorderStyle of
        bsRaised: Frame3d(canvas, LedRect, clWhite, clGray, FBorderWidth);
        bsLowered: Frame3d(canvas, LedRect, clGray, clWhite, FBorderWidth);
      end;
    end;
  end;
end;

constructor TLed.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  FAlignment := laCenter;
  FBorderColor := clGreen;
  FBOrderStyle := bsRaised;
  FBorderWidth := 2;
  FCaption := 'Led';
  FCtl3D := true;
  FFont := TFont.Create;
  FInterval := 0;
  FLedHeight := 10;
  FLedOn := false;
  FLedWidth := 25;
  FMargin := 3;
  FOffColor := clGreen;
  FOnColor := clLime;
  FPhongSize := 2;
  Height := 42;
  Width := 66;
end;

destructor TLed.Destroy;
begin
  FTimer.Free;
  FFont.Free;
  inherited Destroy;
end;

//-------------------------------------------------------------------------------------------
// About-property editor...
//-------------------------------------------------------------------------------------------
procedure TAboutProperty.Edit;
begin
  Application.MessageBox('Led version 1.0'+#13+#13+
                         'This is a freeware component.'+#13+
                         'Copyright  1997 by Alexander G. Meeder'+#13+
                         'E-mail: ameeder@dds.nl'
	                 ,'Led version 1.0', MB_OK+MB_ICONINFORMATION);
end;

function TAboutProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutProperty.GetValue: string;
begin
  Result := '(About TLed)';
end;

procedure Register;
begin
  RegisterComponents('Samples', [TLed]);
  RegisterPropertyEditor(TypeInfo(TAboutProperty), TLed, 'ABOUT', TAboutProperty);
end;

end.
